using System;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using lpsolve55;
using System.IO;

public partial class lp_solve : System.Web.UI.Page
{
    enum formats
    {
        lp, cplex, lindo, xpress, mps, freemps, mathprog, zimpl, lpfml, dimacs
    }

    string Path;

    protected void Page_Load(object sender, EventArgs e)
    {
        try
        {
            Path = Server.MapPath("\\");
        }
        catch
        {
            Path = @"D:\.NET\csharp\ASP.NET\Projects@Home\lp_solve";
        }

        if (!IsPostBack)
        {
            optLP.Checked = true;
            prevFormat.Value = ((int) formats.lp).ToString();
        }
    }

    protected void btnSolve_Click(object sender, EventArgs e)
    {
        Solve();
    }

    private int LoadLPModel(formats format)
    {
        int lp = 0;

        StreamWriter sw = new StreamWriter(Path + "\\lp_solve\\model.txt");
        sw.WriteLine(txtModel.Text);
        sw.Close();

        result.InnerHtml = "";

        switch (format)
        {
            case formats.lp:
                lp = lpsolve.read_LP(Path + "\\lp_solve\\model.txt", 0, "");
                break;
            case formats.cplex:
                lp = lpsolve.read_XLI("xli_CPLEX", Path + "\\lp_solve\\model.txt", "", "", 0);
                break;
            case formats.lindo:
                lp = lpsolve.read_XLI("xli_LINDO", Path + "\\lp_solve\\model.txt", "", "", 0);
                break;
            case formats.xpress:
                lp = lpsolve.read_XLI("xli_XPRESS", Path + "\\lp_solve\\model.txt", "", "", 0);
                break;
            case formats.mps:
                lp = lpsolve.read_MPS(Path + "\\lp_solve\\model.txt", 0);
                break;
            case formats.freemps:
                lp = lpsolve.read_freeMPS(Path + "\\lp_solve\\model.txt", 0);
                break;
            case formats.mathprog:
                lp = lpsolve.read_XLI("xli_MathProg", Path + "\\lp_solve\\model.txt", "", "", 0);
                break;
            case formats.zimpl:
                lp = lpsolve.read_XLI("xli_ZIMPL", Path + "\\lp_solve\\model.txt", "", "", 0);
                break;
            case formats.lpfml:
                lp = lpsolve.read_XLI("xli_LPFML", Path + "\\lp_solve\\model.txt", "", "", 0);
                break;
            case formats.dimacs:
                lp = lpsolve.read_XLI("xli_DIMACS", Path + "\\lp_solve\\model.txt", "", "", 0);
                break;
        }

        return lp;
    }

    private void WriteLPModel(int lp, formats format)
    {
        switch (format)
        {
            case formats.lp:
                lpsolve.write_lp(lp, Path + "\\lp_solve\\model.txt");
                break;
            case formats.cplex:
                lpsolve.set_XLI(lp, "xli_CPLEX");
                lpsolve.write_XLI(lp, Path + "\\lp_solve\\model.txt", "", false);
                break;
            case formats.lindo:
                lpsolve.set_XLI(lp, "xli_LINDO");
                lpsolve.write_XLI(lp, Path + "\\lp_solve\\model.txt", "", false);
                break;
            case formats.xpress:
                lpsolve.set_XLI(lp, "xli_XPRESS");
                lpsolve.write_XLI(lp, Path + "\\lp_solve\\model.txt", "", false);
                break;
            case formats.mps:
                lpsolve.write_mps(lp, Path + "\\lp_solve\\model.txt");
                break;
            case formats.freemps:
                lpsolve.write_freemps(lp, Path + "\\lp_solve\\model.txt");
                break;
            case formats.mathprog:
                lpsolve.set_XLI(lp, "xli_MathProg");
                lpsolve.write_XLI(lp, Path + "\\lp_solve\\model.txt", "", false);
                break;
            case formats.zimpl:
                lpsolve.set_XLI(lp, "xli_ZIMPL");
                lpsolve.write_XLI(lp, Path + "\\lp_solve\\model.txt", "", false);
                break;
            case formats.lpfml:
                lpsolve.set_XLI(lp, "xli_LPFML");
                lpsolve.write_XLI(lp, Path + "\\lp_solve\\model.txt", "", false);
                break;
            case formats.dimacs:
                lpsolve.set_XLI(lp, "xli_DIMACS");
                lpsolve.write_XLI(lp, Path + "\\lp_solve\\model.txt", "", false);
                break;
        }
    }

    private void Solve()
    {
        lpsolve.Init(Path + "\\bin");

#if false
        int major = 0, minor = 0, release = 0, build = 0;

        lpsolve.lp_solve_version(ref major, ref minor, ref release, ref build);
        txtModel.Text = major.ToString() + "." + minor.ToString() + "." + release.ToString() + "." + build.ToString();
#endif

        StreamWriter sw = new StreamWriter(Path + "\\lp_solve\\model.txt");
        sw.WriteLine(txtModel.Text);
        sw.Close();

        result.InnerHtml = "";

        int lp = 0, Nrows, Ncolumns, i;
        lpsolve.lpsolve_return ret;
        bool ok;

        if (optLP.Checked)
            lp = LoadLPModel(formats.lp);
        if (optCPLEX.Checked)
            lp = LoadLPModel(formats.cplex);
        if (optLINDO.Checked)
            lp = LoadLPModel(formats.lindo);
        if (optXPRESS.Checked)
            lp = LoadLPModel(formats.xpress);
        if (optMPS.Checked)
            lp = LoadLPModel(formats.mps);
        if (optFREEMPS.Checked)
            lp = LoadLPModel(formats.freemps);
        if (optMathProg.Checked)
            lp = LoadLPModel(formats.mathprog);
        if (optZIMPL.Checked)
            lp = LoadLPModel(formats.zimpl);
        if (optLPFML.Checked)
            lp = LoadLPModel(formats.lpfml);
        if (optDIMACS.Checked)
            lp = LoadLPModel(formats.dimacs);
        if (lp == 0)
        {
            result.InnerHtml += "Unable to load model in lp_solve. There is probably a syntax error.";
        }
        else
        {
            const int timeout = 5;

            Nrows = lpsolve.get_Nrows(lp);
            Ncolumns = lpsolve.get_Ncolumns(lp);
            result.InnerHtml += "<table>";

            result.InnerHtml += "<tr>";
            result.InnerHtml += "<td>Number of rows:</td><td>" + Nrows.ToString() + "</td>";
            result.InnerHtml += "</tr>";

            result.InnerHtml += "<tr>";
            result.InnerHtml += "<td>Number of columns:</td><td>" + Ncolumns.ToString() + "</td>";
            result.InnerHtml += "</tr>";

            lpsolve.set_timeout(lp, timeout);
            ret = lpsolve.solve(lp);
            ok = (ret == lpsolve.lpsolve_return.OPTIMAL);

            result.InnerHtml += "<tr>";
            result.InnerHtml += "<td>Return code:</td><td>" + ret.ToString() + ((ret == lpsolve.lpsolve_return.TIMEOUT) ? " (a timeout of " + timeout.ToString() + " seconds was set)" : "") + "</td>";
            result.InnerHtml += "</tr>";

            if (ok)
            {
                result.InnerHtml += "<tr>";
                result.InnerHtml += "<td>Time to solve:</td><td>" + lpsolve.time_elapsed(lp).ToString() + " seconds</td>";
                result.InnerHtml += "</tr>";

                result.InnerHtml += "<tr>";
                result.InnerHtml += "<td>Objective value:</td><td>" + lpsolve.get_objective(lp).ToString() + "</td>";
                result.InnerHtml += "</tr>";
            }
            result.InnerHtml += "</table>";

            if (ok)
            {
                /* variables */
                result.InnerHtml += "<br><table><tr><td>Variables:</td></tr></table>";

                result.InnerHtml += "<table border='1'>";

                result.InnerHtml += "<tr>";
                result.InnerHtml += "<th>" + "Name" + "</th><th>" + "Value" + "</th>";
                result.InnerHtml += "</tr>";

                for (i = 1; i <= Ncolumns; i++)
                {
                    result.InnerHtml += "<tr>";
                    result.InnerHtml += "<td>" + lpsolve.get_col_name(lp, i) + "</td><td>" + lpsolve.get_var_primalresult(lp, Nrows + i).ToString() + "</td>";
                    result.InnerHtml += "</tr>";
                }
                result.InnerHtml += "</table>";

                /* constraints */
                result.InnerHtml += "<br><table><tr><td>Constraints:</td></tr></table>";

                result.InnerHtml += "<table border='1'>";

                result.InnerHtml += "<tr>";
                result.InnerHtml += "<th>" + "Name" + "</th><th>" + "Value" + "</th>";
                result.InnerHtml += "</tr>";

                for (i = 1; i <= Nrows; i++)
                {
                    result.InnerHtml += "<tr>";
                    result.InnerHtml += "<td>" + lpsolve.get_row_name(lp, i) + "</td><td>" + lpsolve.get_var_primalresult(lp, i).ToString() + "</td>";
                    result.InnerHtml += "</tr>";
                }
                result.InnerHtml += "</table>";
            }
            lpsolve.delete_lp(lp);
        }
    }

    void LoadModel(string filename)
    {
        result.InnerHtml = "";

        StreamReader sr = new StreamReader(Path + "\\lp_solve\\" + filename);
        txtModel.Text = sr.ReadToEnd();
        sr.Close();
    }

    protected void btnExample_Click(object sender, EventArgs e)
    {
        if (optLP.Checked)
            LoadModel("lp.txt");
        if (optCPLEX.Checked)
            LoadModel("cplex.txt");
        if (optLINDO.Checked)
            LoadModel("lindo.txt");
        if (optXPRESS.Checked)
            LoadModel("xpress.txt");
        if (optMPS.Checked)
            LoadModel("mps.txt");
        if (optFREEMPS.Checked)
            LoadModel("freemps.txt");
        if (optMathProg.Checked)
            LoadModel("mathprog.txt");
        if (optZIMPL.Checked)
            LoadModel("zimpl.txt");
        if (optLPFML.Checked)
            LoadModel("lpfml.txt");
        if (optDIMACS.Checked)
            LoadModel("dimacs.txt");
    }
    protected void Format_Changed(object sender, EventArgs e)
    {
        RadioButton r = (RadioButton)sender;
        int lp;
        formats format = 0;

        if (r == optLP)
            format = formats.lp;
        if (r ==  optCPLEX) 
            format = formats.cplex;
        if (optLINDO.Checked)
            format = formats.lindo;
        if (optXPRESS.Checked)
            format = formats.xpress;
        if (optMPS.Checked)
            format = formats.mps;
        if (optFREEMPS.Checked)
            format = formats.freemps;
        if (optMathProg.Checked)
            format = formats.mathprog;
        if (optZIMPL.Checked)
            format = formats.zimpl;
        if (optLPFML.Checked)
            format = formats.lpfml;
        if (optDIMACS.Checked)
            format = formats.dimacs;

        lp = LoadLPModel((formats)int.Parse(prevFormat.Value));
        prevFormat.Value = ((int)format).ToString();
        if (lp != 0)
        {
            WriteLPModel(lp, (formats)int.Parse(prevFormat.Value));
            lpsolve.delete_lp(lp);
            LoadModel("model.txt");
        }
    }
}
